import aiohttp
from typing import Any, Dict


async def request(
    hub,
    method: str,
    url: str,
    headers: Dict[str, Any],
    base_path: str = "",
    ref: str = "",
    data: Any = None,
    json: Dict[str, Any] = None,
    **kwargs,
):
    full_url = "/".join(x for x in (url, base_path, ref) if x)
    async with aiohttp.ClientSession() as session:
        async with session.request(
            method=method,
            url=full_url,
            headers=headers,
            data=data,
            json=json,
            params=kwargs,
            allow_redirects=True,
            raise_for_status=True,
        ) as response:
            ret = await response.json()
            return ret



from typing import Any, Dict


def _sanitize_params(**kwargs) -> Dict[str, Any]:
    ret = {}
    for k, v in kwargs.items():
        if v is None:
            continue
        elif isinstance(v, bool):
            ret[k] = str(v).lower()
        elif isinstance(v, dict):
            ret[k] = _sanitize_params(**v)
        else:
            ret[k] = v
    return ret


async def pre_request(hub, ctx):
    kwargs = ctx.get_arguments()
    request_params = kwargs.get("kwargs", {})
    clean_params = _sanitize_params(**request_params)
    ctx.kwargs.update(clean_params)


async def call_request(hub, ctx):
    try:
        return await ctx.func(*ctx.args, **ctx.kwargs)
    except Exception as e:
        hub.log.error(f"{e.__class__.__name__}: {e}")
        raise


def post_request(hub, ctx):
    if isinstance(ctx.ret, Dict):
        if "results" in ctx.ret:
            return ctx.ret["results"]
    return ctx.ret



# https://vdc-download.vmware.com/vmwb-repository/dcr-public/7baea724-ebcb-4fba-b604-44cf9fe035ed/6b5f7b84-3be1-4bb3-8894-4e6da76e668e/vrealize-automation-identity-api.json
__func_alias__ = {"list_": "list"}


async def list_(hub, ctx, **kwargs):
    ret = await hub.tool.csp.org.get(ctx, "users", **kwargs)
    return ret["users"]


async def search(hub, ctx, user_search_term: str = "@", **kwargs):
    result = await hub.tool.csp.org.get(
        ctx, f"users/search", userSearchTerm=user_search_term, **kwargs,
    )
    ret = []
    for r in result["results"]:
        user = r.pop("user")
        user.update(r)
        ret.append(user)
    return ret



def __func_alias__(hub):
    """
    Create functions on the hub at this location for common request objects
    """
    aliases = {}
    for method in ("delete", "get", "head", "post", "patch"):
        aliases[method] = _get_caller(hub, method=method)

    return aliases


def _get_caller(hub, method: str):
    async def _request(ctx, ref: str, org_id: str = None, **kwargs):
        """
        Make an authenticated csp request with a url built for a specific org_id
        :param ctx: The profile information generated by the acct plugin (is implicitly passed to exec modules and state modules)
        :param ref: The rest of the url path that follows the "org_id"
        :param org_id: Use this org_id if explicitly passed; fall back to the default in acct profile
        :param kwargs: Any args to pass to the underlying requests library
        :return: The result of the request
        """
        org_id = org_id or ctx.acct.default_org_id
        func = getattr(hub.tool.csp.authed, method.lower())
        return await func(ctx, ref=f"orgs/{org_id}/{ref}", **kwargs,)

    return _request



# https://console.cloud.vmware.com/csp/gateway/am/api/swagger-ui.html
PUBLIC_CSP_URL = "https://console.cloud.vmware.com"


async def gather(hub, profiles):
    """
    Given a refresh token, retrieve an access token.
    If a default_org_id isn't supplied, use the default org of the connected account.
    Any extra parameters will be saved as part of the profile.

    Example:

    .. code-block:: yaml

        csp.token:
          profile_name:
            refresh_token: dmd23q3au8ljyajcvhz207of4ivsn9vjiaxzez223qeagdpe0voqiasknykv58jt
            # optional configuration
            csp_url: https://console.cloud.vmware.com
            default_org_id: my_org_id
            my_kwarg: my_value
    """
    sub_profiles = {}
    for profile, ctx in profiles.get("csp.token", {}).items():
        try:
            # Fall back on to the public CSP url
            ctx["csp_url"] = ctx.get("csp_url", PUBLIC_CSP_URL)

            hub.log.debug(f"connecting to csp with profile: {profile}")
            token_data = await hub.tool.csp.init.post(
                url=ctx["csp_url"],
                ref="auth/api-tokens/authorize",
                refresh_token=ctx["refresh_token"],
            )
            ctx["access_token"] = token_data["access_token"]

            if "default_org_id" not in ctx:
                loggdedin_user_profile = await hub.tool.csp.session.request(
                    method="get",
                    url=ctx["csp_url"],
                    base_path=hub.tool.csp.init.PATH,
                    ref="loggedin/user/profile",
                    headers={
                        "Content-Type": "application/json",
                        "csp-auth-token": ctx["access_token"],
                    },
                )
                ctx["default_org_id"] = loggdedin_user_profile.get("defaultOrgId")

            sub_profiles[profile] = ctx
            hub.log.debug(f"connected to csp with profile: {profile}")
        except Exception as e:
            hub.log.error(f"{e.__class__.__name__}: {e}")
            continue

    return sub_profiles



__func_alias__ = {"list_": "list"}


async def list_(hub, ctx, org_id: str = None):
    ret = await hub.tool.csp.org.get(ctx, "roles", org_id=org_id, expand=True)
    return ret["orgRolesData"]


async def get(hub, ctx, role_id: str, org_id: str = None):
    return await hub.tool.csp.org.get(ctx, f"roles/{role_id}", org_id=org_id)



async def pre(hub, ctx):
    kwargs = ctx.get_arguments()
    func_ctx = kwargs["ctx"]

    if "acct" not in func_ctx:
        raise ConnectionError("No profile found")

    if "access_token" not in func_ctx.acct:
        raise ConnectionError(f"No access token found")



def __func_alias__(hub):
    """
    Create functions on the hub at this location for common request objects
    """
    aliases = {}
    for method in ("delete", "get", "head", "post", "patch"):
        aliases[method] = _get_caller(hub, method=method)

    return aliases


def _get_caller(hub, method: str):
    async def _request(ctx, ref: str, **kwargs):
        """
        Make an authenticated csp request at the given url

        :param ctx: The profile information generated by the acct plugin (is implicitly passed to exec modules and state modules)
        :param ref: The rest of the url path that follows the "org_id"
        :param kwargs: Any args to pass to the underlying requests library
        :return: The result of the request
        """
        return await hub.tool.csp.session.request(
            method=method,
            url=ctx.acct.csp_url,
            base_path=hub.tool.csp.init.PATH,
            ref=ref.strip("/"),
            headers={
                "Content-Type": "application/json",
                "csp-auth-token": ctx.acct.access_token,
            },
            **kwargs,
        )

    return _request
